/*
  Rhino test framework

   Copyright (C) 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.script.ScriptEngineManager;



/**
 * Basic ScriptEngineManager test suite. This test checks basic functionality of
 * a ScriptEngineManager class and should be started as the first test of whole
 * suite, because other tests will fail in case the test defined in this class
 * will fail (so it usually does not make sence to run any other test suites).
 * 
 * @author Pavel Tisnovsky
 */
public class ScriptEngineManagerTest extends BaseRhinoTest {

    @Override
    protected void setUp(String[] args) {
        // empty block
        return;
    }

    @Override
    protected void tearDown() {
        // empty block
        return;
    }

    /**
     * Simple check if it is possible to make instance of ScriptEngineManager.
     */
    protected void testScriptEngineManagerNew() {
        @SuppressWarnings("unused")
        ScriptEngineManager manager = new ScriptEngineManager();
    }

    /**
     * Check if newly created ScriptEngineManager is not null.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerNewNotNull() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        assertNotNull(manager, "new ScriptEngineManager() returns null");
    }

    /**
     * Test if it is possible to create more ScriptEngineManager instances.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testMoreEngineManagers() throws Exception {
        List<ScriptEngineManager> list = new ArrayList<ScriptEngineManager>();
        for (int i = 0; i < 10; i++) {
            list.add(new ScriptEngineManager());
        }
        for (ScriptEngineManager manager : list) {
            assertNotNull(manager, "new ScriptEngineManager() returns null");
        }
    }

    /**
     * Check if ScriptEngineManager can return instance of Binding class.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerGetBindings() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        assertNotNull(manager.getBindings(), "ScriptEngineManager.getBindings() returns null");
    }

    /**
     * Check if ScriptEngineManager can return collection of script engine factories.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerGetEngineFactories() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        assertNotNull(manager.getEngineFactories(), "ScriptEngineManager.getEngineFactories() returns null");
    }

    /**
     * Test if get() method with unknown key returns null.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerGet() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        assertTrue(manager.get("foo") == null, "get(unknown_key) returns not null");
    }

    /**
     * Test if NPE is properly thrown if null key is used in get() method.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerGetWithNullKey() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        try {
            manager.get(null);
        }
        catch (NullPointerException npe) {
            return;
        }
        throw new Exception("NPE did not thrown as expected");
    }

    /**
     * Test of ScriptEngineManager.put() method.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutUsingNotNullKeyNotNullValue() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", "bar");
    }

    /**
     * Test of ScriptEngineManager.put() method if value is null. 
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutUsingNotNullKeyNullValue() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", null);
    }

    /**
     * Test of ScriptEngineManager.put() method if key is null. 
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutUsingNullKeyNotNullValue() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        try {
            manager.put(null, "bar");
        }
        catch (NullPointerException npe) {
            return;
        }
        throw new Exception("NPE did not thrown as expected");
    }

    /**
     * Test of ScriptEngineManager.put() method if key and value is null. 
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutUsingNullKeyNullValue() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        try {
            manager.put(null, null);
        }
        catch (NullPointerException npe) {
            return;
        }
        throw new Exception("NPE did not thrown as expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type. 
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetString() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", "bar");
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertEquals(o, "bar", "get(key) does not return correct value");
        assertType(o, String.class, "String class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetBoolean() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", Boolean.FALSE);
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertEquals(o, Boolean.valueOf(false), "get(key) does not return correct value");
        assertType(o, Boolean.class, "Boolean class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetShort() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", Short.valueOf((short)42));
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertEquals(o, Short.valueOf((short)42), "get(key) does not return correct value");
        assertType(o, Short.class, "Short class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetInteger() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", Integer.valueOf(42));
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertEquals(o, Integer.valueOf(42), "get(key) does not return correct value");
        assertType(o, Integer.class, "Integer class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetLong() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", Long.valueOf(42L));
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertEquals(o, Long.valueOf(42), "get(key) does not return correct value");
        assertType(o, Long.class, "Long class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetFloat() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", Float.valueOf(3.14f));
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertEquals(o, Float.valueOf(3.14f), "get(key) does not return correct value");
        assertType(o, Float.class, "Float class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetDouble() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", Double.valueOf(1.0/3));
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertEquals(o, Double.valueOf(1.0/3), "get(key) does not return correct value");
        assertType(o, Double.class, "Double class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods - maintaining the object type.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetDate() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", new Date());
        Object o = manager.get("foo");
        assertNotNull(o, "get(key) returns null");
        assertType(o, Date.class, "Date class expected");
    }

    /**
     * Test of ScriptEngineManager.put() and get() methods.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerPutGetNull() throws Exception {
        ScriptEngineManager manager = new ScriptEngineManager();
        manager.put("foo", null);
        assertNull(manager.get("foo"), "get(key) returns null");
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new ScriptEngineManagerTest().doTests(args);
    }

}
